// Created on: 1994-02-08
// Created by: Gilles DEBARBOUILLE
// Copyright (c) 1994-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _Quantity_PhysicalQuantity_HeaderFile
#define _Quantity_PhysicalQuantity_HeaderFile

//! List of all physical quantities(Afnor)
enum Quantity_PhysicalQuantity
{
Quantity_MASS,
Quantity_PLANEANGLE,
Quantity_SOLIDANGLE,
Quantity_LENGTH,
Quantity_AREA,
Quantity_VOLUME,
Quantity_SPEED,
Quantity_VELOCITY,
Quantity_ACCELERATION,
Quantity_ANGULARVELOCITY,
Quantity_FREQUENCY,
Quantity_TEMPERATURE,
Quantity_AMOUNTOFSUBSTANCE,
Quantity_DENSITY,
Quantity_MASSFLOW,
Quantity_VOLUMEFLOW,
Quantity_CONSUMPTION,
Quantity_MOMENTUM,
Quantity_KINETICMOMENT,
Quantity_MOMENTOFINERTIA,
Quantity_FORCE,
Quantity_MOMENTOFAFORCE,
Quantity_TORQUE,
Quantity_WEIGHT,
Quantity_PRESSURE,
Quantity_VISCOSITY,
Quantity_KINEMATICVISCOSITY,
Quantity_ENERGY,
Quantity_WORK,
Quantity_POWER,
Quantity_SURFACETENSION,
Quantity_COEFFICIENTOFEXPANSION,
Quantity_THERMALCONDUCTIVITY,
Quantity_SPECIFICHEATCAPACITY,
Quantity_ENTROPY,
Quantity_ENTHALPY,
Quantity_LUMINOUSINTENSITY,
Quantity_LUMINOUSFLUX,
Quantity_LUMINANCE,
Quantity_ILLUMINANCE,
Quantity_LUMINOUSEXPOSITION,
Quantity_LUMINOUSEFFICACITY,
Quantity_ELECTRICCHARGE,
Quantity_ELECTRICCURRENT,
Quantity_ELECTRICFIELDSTRENGTH,
Quantity_ELECTRICPOTENTIAL,
Quantity_ELECTRICCAPACITANCE,
Quantity_MAGNETICFLUX,
Quantity_MAGNETICFLUXDENSITY,
Quantity_MAGNETICFIELDSTRENGTH,
Quantity_RELUCTANCE,
Quantity_RESISTANCE,
Quantity_INDUCTANCE,
Quantity_CAPACITANCE,
Quantity_IMPEDANCE,
Quantity_ADMITTANCE,
Quantity_RESISTIVITY,
Quantity_CONDUCTIVITY,
Quantity_MOLARMASS,
Quantity_MOLARVOLUME,
Quantity_CONCENTRATION,
Quantity_MOLARCONCENTRATION,
Quantity_MOLARITY,
Quantity_SOUNDINTENSITY,
Quantity_ACOUSTICINTENSITY,
Quantity_ACTIVITY,
Quantity_ABSORBEDDOSE,
Quantity_DOSEEQUIVALENT
};

#endif // _Quantity_PhysicalQuantity_HeaderFile
