// Copyright (c) 2015 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _Aspect_FBConfig_HeaderFile
#define _Aspect_FBConfig_HeaderFile

#if !defined(_WIN32) && (!defined(__APPLE__) || defined(MACOSX_USE_GLX)) && !defined(__ANDROID__) && !defined(__QNX__)
  typedef struct __GLXFBConfigRec* GLXFBConfig;
  typedef GLXFBConfig Aspect_FBConfig; // GLXFBConfig* under UNIX
#else
  typedef void* Aspect_FBConfig;       // unused on other systems
#endif

#endif // _Aspect_FBConfig_HeaderFile
