echo off

rem CASDEB comes as third argument

if /I "%VCVER%" == "vc14" (
  if "%ARCH%" == "64" (
    set "TCL_DIR=C:/conda/envs/_build/Library/bin"
    set "TK_DIR=C:/conda/envs/_build/Library/bin"
    set "FREETYPE_DIR=C:/conda/envs/_build/Library/bin"
    set "FREEIMAGE_DIR="
    set "FREEIMAGEPLUS_DIR="
    set "GL2PS_DIR="
    set "TBB_DIR="
    set "VTK_DIR="

    set "TCL_VERSION_WITH_DOT=8.5"
    set "TK_VERSION_WITH_DOT=8.5"

    set "CSF_OCCTBinPath=C:/conda/envs/_build/Library/bin%3"
    set "CSF_OCCTLibPath=C:/conda/envs/_build/Library/lib%3"

    set "CSF_OCCTIncludePath=C:/conda/envs/_build/Library/include/opencascade"
    set "CSF_OCCTResourcePath=C:/conda/envs/_build/Library/share/opencascade/resources"
    set "CSF_OCCTDataPath=C:/conda/envs/_build/Library/share/opencascade/data"
    set "CSF_OCCTSamplesPath=C:/conda/envs/_build/Library/share/opencascade/samples"
    set "CSF_OCCTTestsPath=C:/conda/envs/_build/Library/share/opencascade/tests"
    set "CSF_OCCTDocPath=C:/conda/envs/_build/Library/share/doc/opencascade"

    rem for compatability with external application using CASROOT
    set "CASROOT=C:/conda/envs/_build/Library"
  )
)

