// Created on: 1997-03-04
// Created by: Mister rmi
// Copyright (c) 1997-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _FWOSDriver_DriverFactory_HeaderFile
#define _FWOSDriver_DriverFactory_HeaderFile

#include <Standard.hxx>
#include <Standard_Type.hxx>

#include <CDF_MetaDataDriverFactory.hxx>
class CDF_MetaDataDriver;


class FWOSDriver_DriverFactory;
DEFINE_STANDARD_HANDLE(FWOSDriver_DriverFactory, CDF_MetaDataDriverFactory)


class FWOSDriver_DriverFactory : public CDF_MetaDataDriverFactory
{

public:

  
  Standard_EXPORT FWOSDriver_DriverFactory();
  
  Standard_EXPORT Handle(CDF_MetaDataDriver) Build() const Standard_OVERRIDE;




  DEFINE_STANDARD_RTTIEXT(FWOSDriver_DriverFactory,CDF_MetaDataDriverFactory)

protected:




private:




};







#endif // _FWOSDriver_DriverFactory_HeaderFile
