// Created by: Peter KURNEV
// Copyright (c) 1999-2013 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef BOPCol_NCVector_HeaderFile
#define BOPCol_NCVector_HeaderFile

#include <Standard.hxx>
#include <NCollection_Vector.hxx>
#include <NCollection_BaseAllocator.hxx>

//=======================================================================
//class    : BOPCol_NCVector
//purpose  : 
//=======================================================================
template <class Type> class BOPCol_NCVector 
  :
  public NCollection_Vector<Type>
{
 public:
  BOPCol_NCVector(const Standard_Integer theIncrement=256,
		  const Handle(NCollection_BaseAllocator)& theAlloc = NULL) 
    : NCollection_Vector<Type>(theIncrement, theAlloc)
  {}
  //
  Type& Append1 ()
  {
    Type& anAppended = *(Type*)this->expandV (this->myLength);
    return anAppended;
  }
  //
  Standard_Integer Extent() const
  {
    return this->myLength;
  }
};

#endif
