// Created on: 1995-12-04
// Created by: EXPRESS->CDL V0.2 Translator
// Copyright (c) 1995-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _RWStepAP214_HeaderFile
#define _RWStepAP214_HeaderFile

#include <Standard.hxx>
#include <Standard_DefineAlloc.hxx>
#include <Standard_Handle.hxx>

class RWStepAP214_ReadWriteModule;
class RWStepAP214_GeneralModule;
class RWStepAP214_RWAutoDesignActualDateAndTimeAssignment;
class RWStepAP214_RWAutoDesignActualDateAssignment;
class RWStepAP214_RWAutoDesignApprovalAssignment;
class RWStepAP214_RWAutoDesignDateAndPersonAssignment;
class RWStepAP214_RWAutoDesignGroupAssignment;
class RWStepAP214_RWAutoDesignNominalDateAndTimeAssignment;
class RWStepAP214_RWAutoDesignNominalDateAssignment;
class RWStepAP214_RWAutoDesignOrganizationAssignment;
class RWStepAP214_RWAutoDesignPersonAndOrganizationAssignment;
class RWStepAP214_RWAutoDesignPresentedItem;
class RWStepAP214_RWAutoDesignSecurityClassificationAssignment;
class RWStepAP214_RWAutoDesignDocumentReference;
class RWStepAP214_RWAppliedDateAndTimeAssignment;
class RWStepAP214_RWAppliedDateAssignment;
class RWStepAP214_RWAppliedApprovalAssignment;
class RWStepAP214_RWAppliedGroupAssignment;
class RWStepAP214_RWAppliedOrganizationAssignment;
class RWStepAP214_RWAppliedPersonAndOrganizationAssignment;
class RWStepAP214_RWAppliedPresentedItem;
class RWStepAP214_RWAppliedSecurityClassificationAssignment;
class RWStepAP214_RWAppliedDocumentReference;
class RWStepAP214_RWAppliedExternalIdentificationAssignment;
class RWStepAP214_RWClass;
class RWStepAP214_RWExternallyDefinedClass;
class RWStepAP214_RWExternallyDefinedGeneralProperty;
class RWStepAP214_RWRepItemGroup;



class RWStepAP214 
{
public:

  DEFINE_STANDARD_ALLOC

  
  //! enforced the initialisation of the  libraries
  Standard_EXPORT static void Init();




protected:





private:




friend class RWStepAP214_ReadWriteModule;
friend class RWStepAP214_GeneralModule;
friend class RWStepAP214_RWAutoDesignActualDateAndTimeAssignment;
friend class RWStepAP214_RWAutoDesignActualDateAssignment;
friend class RWStepAP214_RWAutoDesignApprovalAssignment;
friend class RWStepAP214_RWAutoDesignDateAndPersonAssignment;
friend class RWStepAP214_RWAutoDesignGroupAssignment;
friend class RWStepAP214_RWAutoDesignNominalDateAndTimeAssignment;
friend class RWStepAP214_RWAutoDesignNominalDateAssignment;
friend class RWStepAP214_RWAutoDesignOrganizationAssignment;
friend class RWStepAP214_RWAutoDesignPersonAndOrganizationAssignment;
friend class RWStepAP214_RWAutoDesignPresentedItem;
friend class RWStepAP214_RWAutoDesignSecurityClassificationAssignment;
friend class RWStepAP214_RWAutoDesignDocumentReference;
friend class RWStepAP214_RWAppliedDateAndTimeAssignment;
friend class RWStepAP214_RWAppliedDateAssignment;
friend class RWStepAP214_RWAppliedApprovalAssignment;
friend class RWStepAP214_RWAppliedGroupAssignment;
friend class RWStepAP214_RWAppliedOrganizationAssignment;
friend class RWStepAP214_RWAppliedPersonAndOrganizationAssignment;
friend class RWStepAP214_RWAppliedPresentedItem;
friend class RWStepAP214_RWAppliedSecurityClassificationAssignment;
friend class RWStepAP214_RWAppliedDocumentReference;
friend class RWStepAP214_RWAppliedExternalIdentificationAssignment;
friend class RWStepAP214_RWClass;
friend class RWStepAP214_RWExternallyDefinedClass;
friend class RWStepAP214_RWExternallyDefinedGeneralProperty;
friend class RWStepAP214_RWRepItemGroup;

};







#endif // _RWStepAP214_HeaderFile
