// Created on: 2007-04-28
// Created by: Alexander GRIGORIEV
// Copyright (c) 2007-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef BinLDrivers_VectorOfDocumentSection_H
#define BinLDrivers_VectorOfDocumentSection_H

#include <NCollection_Vector.hxx>
#include <BinLDrivers_DocumentSection.hxx>

/// Collection of BinLDrivers_DocumentSection
typedef NCollection_Vector <BinLDrivers_DocumentSection>
                                BinLDrivers_VectorOfDocumentSection;

#endif   // BinLDrivers_VectorOfDocumentSection_H
