// Copyright (c) 2013-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _Image_PixMap_Handle_H__
#define _Image_PixMap_Handle_H__

#include <Standard_Type.hxx>
#include <Standard_Transient.hxx>

class Image_PixMap;
DEFINE_STANDARD_HANDLE(Image_PixMap, Standard_Transient)
typedef Handle(Image_PixMap) Image_PixMap_Handle;

#endif // _Image_PixMap_Handle_H__
