// Created on: 1997-03-19
// Created by: Prestataire Mary FABIEN
// Copyright (c) 1997-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _TestTopOpeTools_Mesure_HeaderFile
#define _TestTopOpeTools_Mesure_HeaderFile

#include <Standard.hxx>
#include <Standard_DefineAlloc.hxx>
#include <Standard_Handle.hxx>

#include <TCollection_AsciiString.hxx>
#include <TColgp_HArray1OfPnt.hxx>
#include <Standard_Integer.hxx>
#include <Standard_Real.hxx>
#include <TColgp_Array1OfPnt.hxx>
class TCollection_AsciiString;
class gp_Pnt;



class TestTopOpeTools_Mesure 
{
public:

  DEFINE_STANDARD_ALLOC

  
  Standard_EXPORT TestTopOpeTools_Mesure();
  
  Standard_EXPORT TestTopOpeTools_Mesure(const TCollection_AsciiString& name);
  
  Standard_EXPORT TestTopOpeTools_Mesure(const Handle(TColgp_HArray1OfPnt)& Pnts);
  
  Standard_EXPORT void Add (const Standard_Integer n, const Standard_Real t);
  
  Standard_EXPORT void SetName (const TCollection_AsciiString& Name);
  
  Standard_EXPORT const TCollection_AsciiString& Name() const;
  
  Standard_EXPORT const TColgp_Array1OfPnt& Pnts() const;
  
  Standard_EXPORT const gp_Pnt& Pnt (const Standard_Integer I) const;
  
  Standard_EXPORT Standard_Integer NPnts() const;
  
  Standard_EXPORT void Clear();




protected:





private:



  TCollection_AsciiString myName;
  Handle(TColgp_HArray1OfPnt) myPnts;
  Standard_Integer myNPnts;


};







#endif // _TestTopOpeTools_Mesure_HeaderFile
