// Created on: 1997-01-22
// Created by: Mister rmi
// Copyright (c) 1997-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _FWOSDriver_HeaderFile
#define _FWOSDriver_HeaderFile

#include <Standard.hxx>
#include <Standard_DefineAlloc.hxx>
#include <Standard_Handle.hxx>

class Standard_Transient;
class Standard_GUID;
class FWOSDriver_DriverFactory;



class FWOSDriver 
{
public:

  DEFINE_STANDARD_ALLOC

  
  //! returns a DriverFactory.
  Standard_EXPORT static const Handle(Standard_Transient)& Factory (const Standard_GUID& aGUID);




protected:





private:




friend class FWOSDriver_DriverFactory;

};







#endif // _FWOSDriver_HeaderFile
