// Created by: NW,JPB,CAL
// Copyright (c) 1991-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _Aspect_HatchStyle_HeaderFile
#define _Aspect_HatchStyle_HeaderFile

//! Definition of all available hatch styles.
//!
//! HS_HORIZONTAL
//! HS_HORIZONTAL_WIDE
//! HS_VERTICAL
//! HS_VERTICAL_WIDE
//! HS_DIAGONAL_45
//! HS_DIAGONAL_45_WIDE
//! HS_DIAGONAL_135
//! HS_DIAGONAL_135_WIDE
//! HS_GRID
//! HS_GRID_WIDE
//! HS_GRID_DIAGONAL
//! HS_GRID_DIAGONAL_WIDE
enum Aspect_HatchStyle
{
Aspect_HS_HORIZONTAL,
Aspect_HS_HORIZONTAL_WIDE,
Aspect_HS_VERTICAL,
Aspect_HS_VERTICAL_WIDE,
Aspect_HS_DIAGONAL_45,
Aspect_HS_DIAGONAL_45_WIDE,
Aspect_HS_DIAGONAL_135,
Aspect_HS_DIAGONAL_135_WIDE,
Aspect_HS_GRID,
Aspect_HS_GRID_WIDE,
Aspect_HS_GRID_DIAGONAL,
Aspect_HS_GRID_DIAGONAL_WIDE
};

#endif // _Aspect_HatchStyle_HeaderFile
