// Created on: 1993-03-31
// Created by: NW,JPB,CAL
// Copyright (c) 1993-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _Graphic3d_NameOfMaterial_HeaderFile
#define _Graphic3d_NameOfMaterial_HeaderFile

//! Types of aspect materials.
enum Graphic3d_NameOfMaterial
{
Graphic3d_NOM_BRASS,
Graphic3d_NOM_BRONZE,
Graphic3d_NOM_COPPER,
Graphic3d_NOM_GOLD,
Graphic3d_NOM_PEWTER,
Graphic3d_NOM_PLASTER,
Graphic3d_NOM_PLASTIC,
Graphic3d_NOM_SILVER,
Graphic3d_NOM_STEEL,
Graphic3d_NOM_STONE,
Graphic3d_NOM_SHINY_PLASTIC,
Graphic3d_NOM_SATIN,
Graphic3d_NOM_METALIZED,
Graphic3d_NOM_NEON_GNC,
Graphic3d_NOM_CHROME,
Graphic3d_NOM_ALUMINIUM,
Graphic3d_NOM_OBSIDIAN,
Graphic3d_NOM_NEON_PHC,
Graphic3d_NOM_JADE,
Graphic3d_NOM_CHARCOAL,
Graphic3d_NOM_WATER,
Graphic3d_NOM_GLASS,
Graphic3d_NOM_DIAMOND,
Graphic3d_NOM_DEFAULT,
Graphic3d_NOM_UserDefined
};

#endif // _Graphic3d_NameOfMaterial_HeaderFile
