// Created on: 1992-11-13
// Created by: GG
// Copyright (c) 1992-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _V3d_TypeOfZclipping_HeaderFile
#define _V3d_TypeOfZclipping_HeaderFile

//! The available options are
//! -   V3d_OFF: Zclipping (clipping executed in the
//! depth of the view) is deactivated,
//! -   V3d_BACK: back Zclipping is activated,
//! -   V3d_FRONT: front Zclipping is activated,
//! -   V3d_SLICE: both front and back Zclippings are activated.
enum V3d_TypeOfZclipping
{
V3d_OFF,
V3d_BACK,
V3d_FRONT,
V3d_SLICE
};

#endif // _V3d_TypeOfZclipping_HeaderFile
