// Created on: 1992-11-13
// Created by: GG
// Copyright (c) 1992-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _V3d_TypeOfOrientation_HeaderFile
#define _V3d_TypeOfOrientation_HeaderFile

//! Determines the type of orientation.
enum V3d_TypeOfOrientation
{
V3d_Xpos,
V3d_Ypos,
V3d_Zpos,
V3d_Xneg,
V3d_Yneg,
V3d_Zneg,
V3d_XposYpos,
V3d_XposZpos,
V3d_YposZpos,
V3d_XnegYneg,
V3d_XnegYpos,
V3d_XnegZneg,
V3d_XnegZpos,
V3d_YnegZneg,
V3d_YnegZpos,
V3d_XposYneg,
V3d_XposZneg,
V3d_YposZneg,
V3d_XposYposZpos,
V3d_XposYnegZpos,
V3d_XposYposZneg,
V3d_XnegYposZpos,
V3d_XposYnegZneg,
V3d_XnegYposZneg,
V3d_XnegYnegZpos,
V3d_XnegYnegZneg
};

#endif // _V3d_TypeOfOrientation_HeaderFile
